﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CoolKidEngine.Menu;
using CoolKidEngine.Utility;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using RitualisticCapitalism.AssetsManagement;

namespace RitualisticCapitalism
{
	public class GameMaster
	{
		protected TileMap GameMap { get; set; }

		List<Building> AvailableBuildings = new List<Building>();
		List<Facility> AvailableFacilities = new List<Facility>();
		List<Item> AvailableItems = new List<Item>();

		List<Player> GamePlayers = new List<Player>();

		int CurrentPlayerTurn = 0;

		Camera2D Camera = null;
		Menu CurrentPlayerMenu = null;
		Point ScreenSize;

		Menu ActiveRightClickMenu = null;

		Menu BuyMenu = null;
		Menu SellMenu = null;

		decimal TurnCounter = 0;

		int buildingCost = 100000; //100k
		int bidIncrement = 100000;

		InteriorTile baseInteriorTile = null;

		//Tile, bid, turns bid unchanged
		Dictionary<LandTile, Tuple<Player, int, int>> ActiveBids = new Dictionary<LandTile, Tuple<Player, int, int>>();

		public GameMaster
			(
				List<BuildingConfig> buildings,
				List<FacilityConfig> facilities,
				List<ItemConfig> items,
				InteriorTile interiorTile,
				Camera2D camera,
				Menu baseMenu,
				Menu gameMenu,
				Point screenSize
			)
		{
			Camera = camera;
			ScreenSize = screenSize;
			CurrentPlayerMenu = (Menu)baseMenu.Clone();
			ActiveRightClickMenu = (Menu)gameMenu.Clone();
			BuyMenu = (Menu)gameMenu.Clone();
			SellMenu = (Menu)gameMenu.Clone();

			baseInteriorTile = interiorTile;

			foreach (var building in buildings)
			{
				Building newBuilding = new Building(building);
				AvailableBuildings.Add(newBuilding);
			}

			foreach(var facility in facilities)
			{
				Facility newFacility = new Facility(facility);
				AvailableFacilities.Add(newFacility);
			}

			foreach(var item in items)
			{
				Item newItem = new Item(item);
				AvailableItems.Add(newItem);
			}
		}

		public void Load(GraphicsDevice device, int buildingSize, int facilitySize, int itemSize, int logoSize)
		{
			foreach(var building in AvailableBuildings)
				building.Load(device, buildingSize);

			foreach(var facility in AvailableFacilities)
				facility.Load(device, facilitySize);

			foreach (var item in AvailableItems)
				item.Load(device, itemSize);
		}

		public void Setup(List<Player> players, TileMap map)
		{
			GamePlayers = players;
			GameMap = map;
			StartPlayerTurn(0);
		}

		public Building GetBuilding(string buildingName)
		{
			return AvailableBuildings.FirstOrDefault(x => x.SourceConfig.Name.ToLower() == buildingName.ToLower());
		}

		Building TargetBuilding = null;
		public void UpdateMouse(MouseState cState, MouseState pState)
		{
			if (GamePlayers[CurrentPlayerTurn].IsAI)
				return;

			CurrentPlayerMenu.ClickButtons(pState, cState);

			if(displayBuyMenu)
			{
				if(BuyMenu.LeftClickLabels(pState, cState))
				{
					BuyMenu.ClickButtons(pState, cState);
				}
				return;
			}

			if(displaySellMenu)
			{
				SellMenu.ClickButtons(pState, cState);
			}

			if(cState.RightButton == ButtonState.Released && pState.RightButton == ButtonState.Pressed)
			{
				//if we're clicking on the target building menu, check for facility tiles
				bool didClickInteriorMenu = false;
				if (TargetBuilding != null)
				{
					didClickInteriorMenu = TargetBuilding.InteriorMenu.RightClickLabels(pState, cState);
				}

				if (didClickInteriorMenu)
				{
					var tile = (InteriorTile)TargetBuilding.InteriorMap.GetTileAtPosition(new Vector2(cState.X, cState.Y));
					SetupInteriorRightClick(cState, tile);
				}
				else
				{
					var tile = (LandTile)GameMap.GetTileAtPosition(Camera.GetMousePos(new Vector2(cState.X, cState.Y)));
					SetupRightClick(cState, tile);
					TargetBuilding = null;
				}
			}

			if(showRightClickMenu)
			{
				ActiveRightClickMenu.ClickButtons(pState, cState, Camera.GetMousePos(new Vector2(cState.X, cState.Y)));
			}
			else if(showInteriorClickMenu)
			{
				ActiveRightClickMenu.ClickButtons(pState, cState, new Vector2(cState.X, cState.Y));
			}

			if (cState.LeftButton == ButtonState.Released && pState.LeftButton == ButtonState.Pressed)
			{
				showRightClickMenu = false;
				showInteriorClickMenu = false;
				Vector2 camMousePos = Camera.GetMousePos(new Vector2(cState.X, cState.Y));

				//Don't change the current building if we clicked on the background label
				bool didClickInteriorMenu = false;
				if (TargetBuilding != null)
				{
					didClickInteriorMenu = TargetBuilding.InteriorMenu.LeftClickLabels(pState, cState);

					var interiorTile = (InteriorTile)TargetBuilding.InteriorMap.GetTileAtPosition(new Vector2(cState.X, cState.Y));
					TargetInteriorTile = interiorTile;
					if (TargetInteriorTile.Facility != null && TargetInteriorTile.Facility.SourceConfig.Name.Contains("Sell"))
					{
						SetupSellMenu(TargetInteriorTile);
						return;
					}
					else
					{
						//displayBuyMenu = false;
						//displaySellMenu = false;
					}
				}

				var tile = (LandTile)GameMap.GetTileAtPosition(camMousePos);
				if (!didClickInteriorMenu 
					&& CoolMath.GetPointDistance(camMousePos, tile.Center) < tile.DestRectangle.Width)
				{
					if (tile.Owner == GamePlayers[CurrentPlayerTurn] && tile.Building != null)
					{
						TargetBuilding = tile.Building;
						displayBuyMenu = false;
						displaySellMenu = false;
					}
					else
					{
						TargetBuilding = null;
						displayBuyMenu = false;
						displaySellMenu = false;
					}
				}
				else if(!didClickInteriorMenu)
				{
					TargetBuilding = null;
					displayBuyMenu = false;
					displaySellMenu = false;
				}
			}
		}

		public void ModifyPrice(InteriorTile tile, int price)
		{
			tile.ItemPrice += price;
			SetupSellMenu(tile);
		}

		bool displaySellMenu = false;
		private void SetupSellMenu(InteriorTile targetInteriorTile)
		{
			SellMenu.ClearMenu();
			Vector2 start = new Vector2(ScreenSize.X * 0.85f, ScreenSize.Y * 0.7f);
			SellMenu.AddLabel(targetInteriorTile.ItemPrice.ToString("000000"), start + new Vector2(30, 0));
			SellMenu.AddButton("<<<", new Rectangle((int)start.X - 60, (int)start.Y + 20, 30, 20), () => ModifyPrice(targetInteriorTile, -100));
			SellMenu.AddButton("<<", new Rectangle((int)start.X - 20, (int)start.Y + 20, 30, 20), () => ModifyPrice(targetInteriorTile, -10));
			SellMenu.AddButton("<", new Rectangle((int)start.X + 20, (int)start.Y + 20, 30, 20), () => ModifyPrice(targetInteriorTile, -1));

			SellMenu.AddButton(">", new Rectangle((int)start.X + 60, (int)start.Y + 20, 30, 20), () => ModifyPrice(targetInteriorTile, 1));
			SellMenu.AddButton(">>", new Rectangle((int)start.X + 100, (int)start.Y + 20, 30, 20), () => ModifyPrice(targetInteriorTile, 10));
			SellMenu.AddButton(">>>", new Rectangle((int)start.X + 140, (int)start.Y + 20, 30, 20), () => ModifyPrice(targetInteriorTile, 100));
			displaySellMenu = true;
		}

		InteriorTile TargetInteriorTile = null;
		bool showInteriorClickMenu = false;
		private void SetupInteriorRightClick(MouseState cState, InteriorTile tile)
		{
			showInteriorClickMenu = true;
			TargetInteriorTile = tile;

			if (TargetInteriorTile.Facility != null)
			{
				var config = (FacilityConfig)TargetInteriorTile.Facility.SourceConfig;
				List<Item> items;
				if (config.Categories.Contains("All"))
					items = AvailableItems;
				else
					items = AvailableItems.Where(x => config.Categories
						.Contains(((ItemConfig)x.SourceConfig).Category)).ToList();

				ActiveRightClickMenu.ClearMenu();
				ActiveRightClickMenu.AddButton("Remove Facility", Vector2.Zero, () => RemoveFacility(TargetInteriorTile));
				ActiveRightClickMenu.AddLabel(config.Type, Vector2.Zero);
				if (config.Type != "Buy")
				{
					foreach (var item in items)
					{
						ActiveRightClickMenu.AddButton(item.SourceConfig.Name, Vector2.Zero, () => AddItemToFacility(item, TargetInteriorTile));
					}
				}
				else
					ActiveRightClickMenu.AddButton("Buy Item", Vector2.Zero, () => ShowBuyMenu(TargetBuilding, TargetInteriorTile));
				ActiveRightClickMenu.AlignButtonsAndLabelsAsDropdown(TargetInteriorTile.Center, false);
			}
			else if (TargetInteriorTile.Facility == null)
			{
				var config = (BuildingConfig)TargetBuilding.SourceConfig;
				var facilities = AvailableFacilities.Where(x => config.FacilityNames
					.Contains(x.SourceConfig.Name));

				ActiveRightClickMenu.ClearMenu();
				foreach (var facility in facilities)
				{
					ActiveRightClickMenu.AddButton(facility.SourceConfig.Name, Vector2.Zero, () => AddFacility(TargetInteriorTile, facility));
				}
				ActiveRightClickMenu.AlignButtonsAndLabelsAsDropdown(TargetInteriorTile.Center, false);
			}
			else
				showInteriorClickMenu = false;
        }

		bool displayBuyMenu = false;
		public void ShowBuyMenu(Building targetBuilding, InteriorTile tile)
		{
			displayBuyMenu = true;
			//Create a menu with a large panel that covers the whole screen
			//Have a cancel button somewhere so we can get out of it if we need to
			//List all items for sale in the "for sale" list thing
			//Each item in the list will display sell price and shipping cost
			//There will also be a button next to each item to select it
			//Once you select one, remove the menu and set the interior tile to that player's facility or something
			List<Tuple<Player, InteriorTile, LandTile>> SellFacilities = new List<Tuple<Player, InteriorTile, LandTile>>();
			foreach (var landTile in GameMap.m_TileList.Select(x => (LandTile)x)
				.Where(x => x.Owner != null))
			{
				var sellFacilities = landTile.Building.InteriorMap.m_TileList
					.Select(x => (InteriorTile)x)
					.Where(x=>x.Item != null)
					.Where(x => x.Facility.SourceConfig.Name == "Sell");
				foreach (var sell in sellFacilities)
					SellFacilities.Add(new Tuple<Player, InteriorTile, LandTile>(landTile.Owner, sell, landTile));
			}

			BuyMenu.ClearMenu();
			BuyMenu.AddPanel(new Vector2(ScreenSize.X / 2, ScreenSize.Y / 2), new Vector2(ScreenSize.X, ScreenSize.Y), Color.Black);
			BuyMenu.AddLabel("Buy Menu", new Vector2(ScreenSize.X * 0.1f, ScreenSize.Y * 0.1f));
			int count = 0;
			foreach (var sell in SellFacilities)
			{
				int shippingCost = GetShippingCost(targetBuilding.Center, sell.Item3.Center);
                string text = string.Format("{0} Item: {1}    Cost ${2:N0} Shipping ${3:N0}",
					sell.Item1.CorperateConfig.Name, sell.Item2.Item.SourceConfig.Name,
					sell.Item2.ItemPrice, shippingCost);
				int height = (int)(ScreenSize.Y * 0.05f);
				int start = (int)(ScreenSize.Y * 0.15f);
                BuyMenu.AddButton(text, new Rectangle(0, start + (count * height), ScreenSize.X, height), () => SetBuySource(sell.Item2, tile));
				count++;
			}
			BuyMenu.AddButton("Cancel", new Vector2(ScreenSize.X * 0.95f, ScreenSize.Y * 0.1f), () => { displayBuyMenu = false; });
		}

		public void SetBuySource(InteriorTile source, InteriorTile target)
		{
			target.SetBuySource(source);
			displayBuyMenu = false;
		}

		private int GetShippingCost(Vector2 center1, Vector2 center2)
		{
			double dis = CoolMath.GetPointDistance(center1, center2);
			return (int)(10 + (dis * 0.25));
		}

		public void AddFacility(InteriorTile tile, Facility facility)
		{
			tile.SetFacility(facility);
		}

		public void AddItemToFacility(Item item, InteriorTile tile)
		{
			if (tile.Item != null && item.SourceConfig.Name != tile.Item.SourceConfig.Name)
				tile.SetItem(item, 0);
			else if (tile.Item == null)
				tile.SetItem(item, 0);
		}

		public void RemoveFacility(InteriorTile tile)
		{
			tile.RemoveFacility();
		}

		bool showRightClickMenu = false;
		LandTile TargetLandTile = null;
		public void SetupRightClick(MouseState cState, LandTile tile)
		{
			showRightClickMenu = true;
			TargetLandTile = tile;

			if(TargetLandTile.Owner == null && TargetLandTile.Building == null)
			{
				ActiveRightClickMenu.ClearMenu();
				int currentBid = 0;
				string currentHolder = "None";
				if (ActiveBids.ContainsKey(TargetLandTile))
				{
					currentHolder = ActiveBids[TargetLandTile].Item1.CorperateConfig.Name;
                    currentBid = ActiveBids[TargetLandTile].Item2;
				}
				currentBid += bidIncrement;

				ActiveRightClickMenu.AddLabel(currentHolder, Vector2.Zero);
				string nextBid = string.Format("Place Bid: {0:N0}", currentBid);
				if(currentBid >= 1000000)//1mil
					nextBid = string.Format("Place Bid: {0:N1}mil", (decimal)currentBid/1000000m);
				ActiveRightClickMenu.AddButton(nextBid, Vector2.Zero, () => PlaceBid());
				ActiveRightClickMenu.AlignButtonsAndLabelsAsDropdown(TargetLandTile.Center);
			}
			else if(TargetLandTile.Owner == GamePlayers[CurrentPlayerTurn]
				&& TargetLandTile.Building == null)
			{
				ActiveRightClickMenu.ClearMenu();
				foreach(var building in AvailableBuildings
					.Where(x=>((BuildingConfig)x.SourceConfig).IsBuildable))
				{
					ActiveRightClickMenu.AddButton
					(
						string.Format(building.SourceConfig.Name + "| ${0:N0}k", buildingCost / 1000m),
						Vector2.Zero, () => BuildBuilding(building)
					);
                }
				ActiveRightClickMenu.AlignButtonsAndLabelsAsDropdown(TargetLandTile.Center);
			}
			else
			{
				showRightClickMenu = false;
            }
		}

		public void PlaceBid()
		{
			int currentBid = 0;
			if (ActiveBids.ContainsKey(TargetLandTile))
				currentBid = ActiveBids[TargetLandTile].Item2;
			else
				ActiveBids.Add(TargetLandTile, new Tuple<Player, int, int>(GamePlayers[CurrentPlayerTurn], currentBid, 0));

			ActiveBids[TargetLandTile] = new Tuple<Player, int, int>(GamePlayers[CurrentPlayerTurn], currentBid + bidIncrement, 0);
		}

		public void BuildBuilding(Building building)
		{
			Vector2 startPos = new Vector2(ScreenSize.X * 0.85f, ScreenSize.Y * 0.2f);
			Vector2 titlePos = startPos + new Vector2(ScreenSize.X * 0.05f, ScreenSize.Y * 0.05f);
			Vector2 gridStart = titlePos + new Vector2(-ScreenSize.X * 0.025f, ScreenSize.Y * 0.1f);
			Rectangle buildingGrid = new Rectangle((int)gridStart.X, (int)gridStart.Y, 150, 150);

			Menu buildingMenu = (Menu)ActiveRightClickMenu.Clone();
			buildingMenu.ClearMenu();
			buildingMenu.AddPanel(
				startPos + new Vector2(ScreenSize.X * 0.05f, ScreenSize.Y * 0.25f), 
				new Vector2(ScreenSize.X * 0.2f, ScreenSize.Y * 0.7f),
				Color.Black);
			buildingMenu.AddLabel(building.SourceConfig.Name, titlePos);

			//building.SetupBuilding(baseInteriorTile, buildingGrid, buildingMenu);

			TargetLandTile.SetBuilding(building, baseInteriorTile, buildingGrid, buildingMenu, 7);
			GamePlayers[CurrentPlayerTurn].Money -= buildingCost;
			GamePlayers[CurrentPlayerTurn].MoneyHistory.Add(-buildingCost);
		}

		public void EndPlayerTurn()
		{
			int nextPlayer = CurrentPlayerTurn + 1;
			if (nextPlayer >= GamePlayers.Count)
				nextPlayer = 0;

			//Check if bid has been winning for full turn
			//When a bid is won, set the tile color
			List<LandTile> bidsToRemove = new List<LandTile>();
			foreach(var bid in ActiveBids
				.Where(x=>x.Value.Item3 >= GamePlayers.Count - 1))
			{
				bid.Key.Owner = bid.Value.Item1;
				bid.Key.Color = bid.Value.Item1.CorperateConfig.CorperateColor;
				bid.Key.Owner.Money -= bid.Value.Item2;
				bid.Key.Owner.MoneyHistory.Add(-(decimal)bid.Value.Item2);
				bidsToRemove.Add(bid.Key);
			}

			foreach(var tile in bidsToRemove)
			{
				ActiveBids.Remove(tile);
			}

			foreach(var bid in ActiveBids
				.Where(x=>x.Value.Item3 < GamePlayers.Count)
				.ToArray())
			{
				ActiveBids[bid.Key] = new Tuple<Player, int, int>(bid.Value.Item1, bid.Value.Item2,
					bid.Value.Item3 + 1);
			}

			TurnCounter++;

			StartPlayerTurn(nextPlayer);
		}

		protected void StartPlayerTurn(int nextPlayerTurn)
		{
			CurrentPlayerTurn = nextPlayerTurn;
			var currentPlayer = GamePlayers[CurrentPlayerTurn];

			CurrentPlayerMenu.ClearMenu();
			CurrentPlayerMenu.BaseLabel.Color = currentPlayer.CorperateConfig.CorperateColor;
			var namePos = new Vector2(ScreenSize.X * 0.1f, ScreenSize.Y * 0.95f);
			CurrentPlayerMenu.AddLabel
				(currentPlayer.CorperateConfig.Name, namePos);
			var endTurnPos = new Vector2(ScreenSize.X * 0.9f, ScreenSize.Y * 0.90f);
			CurrentPlayerMenu.AddButton
				("End Turn", endTurnPos, () => EndPlayerTurn());
			var turnCounterPos = new Vector2(ScreenSize.X * 0.9f, ScreenSize.Y * 0.05f);
			CurrentPlayerMenu.AddLabel(
				string.Format("Turn: {0:N2}", (TurnCounter/GamePlayers.Count)), turnCounterPos);
			var moneyPos = new Vector2(ScreenSize.X * 0.4f, ScreenSize.Y * 0.95f);
			CurrentPlayerMenu.AddLabel("$" + string.Format("{0:N0}", currentPlayer.Money), moneyPos);

			var landTiles = GameMap.m_TileList.Select(x => (LandTile)x)
				.Where(x => x.Owner == currentPlayer);

#warning figure out how to do facility actions. Mine, manufaction, buy -> sell
#warning then figure out how to sell to people
#warning then figure out AI
		}

		public void Update(GameTime time)
		{
			if(GamePlayers[CurrentPlayerTurn].IsAI)
			{
				EndPlayerTurn();//replace this later
			}
		}

		public void Draw(SpriteBatch sb, GameTime time)
		{
			sb.Begin(SpriteSortMode.Deferred, BlendState.NonPremultiplied, null, null, null, null, Camera.GetTransformation());
			GameMap.Draw(sb, time);
			foreach(var bid in ActiveBids)
			{
				bid.Value.Item1.DrawLogo(bid.Key.Position, sb, time);
			}
			if (showRightClickMenu)
				ActiveRightClickMenu.Draw(sb, time);
			sb.End();

			//Draw Menus
			sb.Begin();
			CurrentPlayerMenu.Draw(sb, time);

			if(TargetBuilding != null)
			{
				TargetBuilding.InteriorMenu.Draw(sb, time);
				TargetBuilding.InteriorMap.Draw(sb, time);
			}

			if (showInteriorClickMenu)
				ActiveRightClickMenu.Draw(sb, time);

			if (displayBuyMenu)
				BuyMenu.Draw(sb, time);

			if (displaySellMenu)
				SellMenu.Draw(sb, time);
			sb.End();
		}
	}
}
